@@ -703,6 +703,16 @@ typedef enum { /*< flags prefix=CLUTTER_EVENT >*/
  * @CLUTTER_DESTROY_NOTIFY: Destroy notification event
  * @CLUTTER_CLIENT_MESSAGE: Client message event
  * @CLUTTER_DELETE: Stage delete event
+ * @CLUTTER_TOUCH_BEGIN: A new touch event sequence has started;
+ *   event added in 1.10
+ * @CLUTTER_TOUCH_UPDATE: A touch event sequence has been updated;
+ *   event added in 1.10
+ * @CLUTTER_TOUCH_END: A touch event sequence has finished;
+ *   event added in 1.10
+ * @CLUTTER_TOUCH_CANCEL: A touch event sequence has been canceled;
+ *   event added in 1.10
+ * @CLUTTER_EVENT_LAST: Marks the end of the #ClutterEventType enumeration;
+ *   added in 1.10
  *
  * Types of events.
  *
@@ -721,7 +731,13 @@ typedef enum { /*< prefix=CLUTTER >*/
   CLUTTER_STAGE_STATE,
   CLUTTER_DESTROY_NOTIFY,
   CLUTTER_CLIENT_MESSAGE,
-  CLUTTER_DELETE
+  CLUTTER_DELETE,
+  CLUTTER_TOUCH_BEGIN,
+  CLUTTER_TOUCH_UPDATE,
+  CLUTTER_TOUCH_END,
+  CLUTTER_TOUCH_CANCEL,
+
+  CLUTTER_EVENT_LAST            /* helper */
 } ClutterEventType;
 
 /**
@@ -730,16 +746,21 @@ typedef enum { /*< prefix=CLUTTER >*/
  * @CLUTTER_SCROLL_DOWN: Scroll down
  * @CLUTTER_SCROLL_LEFT: Scroll left
  * @CLUTTER_SCROLL_RIGHT: Scroll right
+ * @CLUTTER_SCROLL_SMOOTH: Precise scrolling delta (available in 1.10)
  *
  * Direction of a pointer scroll event.
  *
+ * The %CLUTTER_SCROLL_SMOOTH value implies that the #ClutterScrollEvent
+ * has precise scrolling delta information.
+ *
  * Since: 0.4
  */
 typedef enum { /*< prefix=CLUTTER_SCROLL >*/
   CLUTTER_SCROLL_UP,
   CLUTTER_SCROLL_DOWN,
   CLUTTER_SCROLL_LEFT,
-  CLUTTER_SCROLL_RIGHT
+  CLUTTER_SCROLL_RIGHT,
+  CLUTTER_SCROLL_SMOOTH
 } ClutterScrollDirection;
 
 /**
@@ -1094,6 +1115,80 @@ typedef enum {
   CLUTTER_ACTOR_ALIGN_END
 } ClutterActorAlign;
 
+/**
+ * ClutterRepaintFlags:
+ * @CLUTTER_REPAINT_FLAGS_PRE_PAINT: Run the repaint function prior to
+ *   painting the stages
+ * @CLUTTER_REPAINT_FLAGS_POST_PAINT: Run the repaint function after
+ *   painting the stages
+ * @CLUTTER_REPAINT_FLAGS_QUEUE_REDRAW_ON_ADD: Ensure that a new frame
+ *   is queued after adding the repaint function
+ *
+ * Flags to pass to clutter_threads_add_repaint_func_full().
+ *
+ * Since: 1.10
+ */
+typedef enum {
+  CLUTTER_REPAINT_FLAGS_PRE_PAINT = 1 << 0,
+  CLUTTER_REPAINT_FLAGS_POST_PAINT = 1 << 1,
+  CLUTTER_REPAINT_FLAGS_QUEUE_REDRAW_ON_ADD = 1 << 2
+} ClutterRepaintFlags;
+
+/**
+ * ClutterContentGravity:
+ * @CLUTTER_CONTENT_GRAVITY_TOP_LEFT: Align the content to the top left corner
+ * @CLUTTER_CONTENT_GRAVITY_TOP: Align the content to the top edge
+ * @CLUTTER_CONTENT_GRAVITY_TOP_RIGHT: Align the content to the top right corner
+ * @CLUTTER_CONTENT_GRAVITY_LEFT: Align the content to the left edge
+ * @CLUTTER_CONTENT_GRAVITY_CENTER: Align the content to the center
+ * @CLUTTER_CONTENT_GRAVITY_RIGHT: Align the content to the right edge
+ * @CLUTTER_CONTENT_GRAVITY_BOTTOM_LEFT: Align the content to the bottom left corner
+ * @CLUTTER_CONTENT_GRAVITY_BOTTOM: Align the content to the bottom edge
+ * @CLUTTER_CONTENT_GRAVITY_BOTTOM_RIGHT: Align the content to the bottom right corner
+ * @CLUTTER_CONTENT_GRAVITY_RESIZE_FILL: Resize the content to fill the allocation
+ * @CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT: Resize the content to remain within the
+ *   allocation, while maintaining the aspect ratio
+ *
+ * Controls the alignment of the #ClutterContent inside a #ClutterActor.
+ *
+ * Since: 1.10
+ */
+typedef enum {
+  CLUTTER_CONTENT_GRAVITY_TOP_LEFT,
+  CLUTTER_CONTENT_GRAVITY_TOP,
+  CLUTTER_CONTENT_GRAVITY_TOP_RIGHT,
+
+  CLUTTER_CONTENT_GRAVITY_LEFT,
+  CLUTTER_CONTENT_GRAVITY_CENTER,
+  CLUTTER_CONTENT_GRAVITY_RIGHT,
+
+  CLUTTER_CONTENT_GRAVITY_BOTTOM_LEFT,
+  CLUTTER_CONTENT_GRAVITY_BOTTOM,
+  CLUTTER_CONTENT_GRAVITY_BOTTOM_RIGHT,
+
+  CLUTTER_CONTENT_GRAVITY_RESIZE_FILL,
+  CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT
+} ClutterContentGravity;
+
+/**
+ * ClutterScalingFilter:
+ * @CLUTTER_SCALING_FILTER_LINEAR: Linear interpolation filter
+ * @CLUTTER_SCALING_FILTER_NEAREST: Nearest neighbor interpolation filter
+ * @CLUTTER_SCALING_FILTER_TRILINEAR: Trilinear minification filter, with
+ *   mipmap generation; this filter linearly interpolates on every axis,
+ *   as well as between mipmap levels.
+ *
+ * The scaling filters to be used with the #ClutterActor:minification-filter
+ * and #ClutterActor:magnification-filter properties.
+ *
+ * Since: 1.10
+ */
+typedef enum {
+  CLUTTER_SCALING_FILTER_LINEAR,
+  CLUTTER_SCALING_FILTER_NEAREST,
+  CLUTTER_SCALING_FILTER_TRILINEAR
+} ClutterScalingFilter;
+
 G_END_DECLS
 
 #endif /* __CLUTTER_ENUMS_H__ */
