@@ -49,7 +49,12 @@ typedef struct _ClutterEventPrivate {
   ClutterInputDevice *device;
   ClutterInputDevice *source_device;
 
+  gdouble delta_x;
+  gdouble delta_y;
+
   gpointer platform_data;
+
+  guint is_pointer_emulated : 1;
 } ClutterEventPrivate;
 
 static GHashTable *all_events = NULL;
@@ -105,6 +110,16 @@ _clutter_event_set_platform_data (ClutterEvent *event,
   ((ClutterEventPrivate *) event)->platform_data = data;
 }
 
+void
+_clutter_event_set_pointer_emulated (ClutterEvent *event,
+                                     gboolean      is_emulated)
+{
+  if (!is_event_allocated (event))
+    return;
+
+  ((ClutterEventPrivate *) event)->is_pointer_emulated = !!is_emulated;
+}
+
 /**
  * clutter_event_type:
  * @event: a #ClutterEvent
@@ -182,6 +197,12 @@ clutter_event_get_state (const ClutterEvent *event)
     case CLUTTER_BUTTON_RELEASE:
       return event->button.modifier_state;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      return event->touch.modifier_state;
+
     case CLUTTER_MOTION:
       return event->motion.modifier_state;
 
@@ -226,6 +247,13 @@ clutter_event_set_state (ClutterEvent        *event,
       event->motion.modifier_state = state;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      event->touch.modifier_state = state;
+      break;
+
     case CLUTTER_SCROLL:
       event->scroll.modifier_state = state;
       break;
@@ -265,6 +293,7 @@ clutter_event_get_coords (const ClutterEvent *event,
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+    case CLUTTER_EVENT_LAST:
       break;
 
     case CLUTTER_ENTER:
@@ -284,6 +313,14 @@ clutter_event_get_coords (const ClutterEvent *event,
       event_y = event->motion.y;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      event_x = event->touch.x;
+      event_y = event->touch.y;
+      break;
+
     case CLUTTER_SCROLL:
       event_x = event->scroll.x;
       event_y = event->scroll.y;
@@ -323,6 +360,7 @@ clutter_event_set_coords (ClutterEvent *event,
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+    case CLUTTER_EVENT_LAST:
       break;
 
     case CLUTTER_ENTER:
@@ -342,6 +380,14 @@ clutter_event_set_coords (ClutterEvent *event,
       event->motion.y = y;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      event->touch.x = x;
+      event->touch.y = y;
+      break;
+
     case CLUTTER_SCROLL:
       event->scroll.x = x;
       event->scroll.y = y;
@@ -513,6 +559,72 @@ clutter_event_set_related (ClutterEvent *event,
   event->crossing.related = actor;
 }
 
+/**
+ * clutter_event_set_scroll_delta:
+ * @event: a #ClutterEvent of type %CLUTTER_SCROLL
+ * @dx: delta on the horizontal axis
+ * @dy: delta on the vertical axis
+ *
+ * Sets the precise scrolling information of @event.
+ *
+ * Since: 1.10
+ */
+void
+clutter_event_set_scroll_delta (ClutterEvent *event,
+                                gdouble       dx,
+                                gdouble       dy)
+{
+  g_return_if_fail (event != NULL);
+  g_return_if_fail (event->type == CLUTTER_SCROLL);
+
+  if (!is_event_allocated (event))
+    return;
+
+  event->scroll.direction = CLUTTER_SCROLL_SMOOTH;
+
+  ((ClutterEventPrivate *) event)->delta_x = dx;
+  ((ClutterEventPrivate *) event)->delta_y = dy;
+}
+
+/**
+ * clutter_event_get_scroll_delta:
+ * @event: a #ClutterEvent of type %CLUTTER_SCROLL
+ * @dx: (out): return location for the delta on the horizontal axis
+ * @dy: (out): return location for the delta on the vertical axis
+ *
+ * Retrieves the precise scrolling information of @event.
+ *
+ * The @event has to have a #ClutterScrollEvent.direction value
+ * of %CLUTTER_SCROLL_SMOOTH.
+ *
+ * Since: 1.10
+ */
+void
+clutter_event_get_scroll_delta (const ClutterEvent *event,
+                                gdouble            *dx,
+                                gdouble            *dy)
+{
+  gdouble delta_x, delta_y;
+
+  g_return_if_fail (event != NULL);
+  g_return_if_fail (event->type == CLUTTER_SCROLL);
+  g_return_if_fail (event->scroll.direction == CLUTTER_SCROLL_SMOOTH);
+
+  delta_x = delta_y = 0;
+
+  if (is_event_allocated (event))
+    {
+      delta_x = ((ClutterEventPrivate *) event)->delta_x;
+      delta_y = ((ClutterEventPrivate *) event)->delta_y;
+    }
+
+  if (dx != NULL)
+    *dx = delta_x;
+
+  if (dy != NULL)
+    *dy = delta_y;
+}
+
 /**
  * clutter_event_get_scroll_direction:
  * @event: a #ClutterEvent of type %CLUTTER_SCROLL
@@ -743,6 +855,32 @@ clutter_event_set_key_unicode (ClutterEvent *event,
   event->key.unicode_value = key_unicode;
 }
 
+/**
+ * clutter_event_get_event_sequence:
+ * @event: a #ClutterEvent of type %CLUTTER_TOUCH_BEGIN,
+ *   %CLUTTER_TOUCH_UPDATE, %CLUTTER_TOUCH_END, or
+ *   %CLUTTER_TOUCH_CANCEL
+ *
+ * Retrieves the #ClutterEventSequence of @event.
+ *
+ * Return value: (transfer none): the event sequence, or %NULL
+ *
+ * Since: 1.10
+ */
+ClutterEventSequence *
+clutter_event_get_event_sequence (const ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, NULL);
+
+  if (event->type == CLUTTER_TOUCH_BEGIN ||
+      event->type == CLUTTER_TOUCH_UPDATE ||
+      event->type == CLUTTER_TOUCH_END ||
+      event->type == CLUTTER_TOUCH_CANCEL)
+    return event->touch.sequence;
+
+  return NULL;
+}
+
 /**
  * clutter_event_get_device_id:
  * @event: a clutter event 
@@ -821,6 +959,7 @@ clutter_event_set_device (ClutterEvent       *event,
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+    case CLUTTER_EVENT_LAST:
       break;
 
     case CLUTTER_ENTER:
@@ -841,6 +980,13 @@ clutter_event_set_device (ClutterEvent       *event,
       event->scroll.device = device;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      event->touch.device = device;
+      break;
+
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
       event->key.device = device;
@@ -885,6 +1031,7 @@ clutter_event_get_device (const ClutterEvent *event)
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+    case CLUTTER_EVENT_LAST:
       break;
 
     case CLUTTER_ENTER:
@@ -905,6 +1052,13 @@ clutter_event_get_device (const ClutterEvent *event)
       device = event->scroll.device;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      device = event->touch.device;
+      break;
+
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
       device = event->key.device;
@@ -970,6 +1124,8 @@ clutter_event_copy (const ClutterEvent *event)
 
       new_real_event->device = real_event->device;
       new_real_event->source_device = real_event->source_device;
+      new_real_event->delta_x = real_event->delta_x;
+      new_real_event->delta_y = real_event->delta_y;
     }
 
   device = clutter_event_get_device (event);
@@ -997,6 +1153,15 @@ clutter_event_copy (const ClutterEvent *event)
                                            sizeof (gdouble) * n_axes);
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      if (event->touch.axes != NULL)
+        new_event->touch.axes = g_memdup (event->motion.axes,
+                                          sizeof (gdouble) * n_axes);
+      break;
+
     default:
       break;
     }
@@ -1037,6 +1202,13 @@ clutter_event_free (ClutterEvent *event)
           g_free (event->scroll.axes);
           break;
 
+        case CLUTTER_TOUCH_BEGIN:
+        case CLUTTER_TOUCH_UPDATE:
+        case CLUTTER_TOUCH_END:
+        case CLUTTER_TOUCH_CANCEL:
+          g_free (event->touch.axes);
+          break;
+
         default:
           break;
         }
@@ -1301,6 +1473,7 @@ clutter_event_get_axes (const ClutterEvent *event,
     case CLUTTER_LEAVE:
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
+    case CLUTTER_EVENT_LAST:
       break;
 
     case CLUTTER_SCROLL:
@@ -1312,6 +1485,13 @@ clutter_event_get_axes (const ClutterEvent *event,
       retval = event->button.axes;
       break;
 
+    case CLUTTER_TOUCH_BEGIN:
+    case CLUTTER_TOUCH_UPDATE:
+    case CLUTTER_TOUCH_END:
+    case CLUTTER_TOUCH_CANCEL:
+      retval = event->touch.axes;
+      break;
+
     case CLUTTER_MOTION:
       retval = event->motion.axes;
       break;
