@@ -1259,18 +1259,7 @@ clutter_path_node_distance (const ClutterKnot *start,
   t = (end->x - start->x) * (end->x - start->x) +
     (end->y - start->y) * (end->y - start->y);
 
-  /*
-   * If we are using limited precision sqrti implementation, fallback on
-   * clib sqrt if the precission would be less than 10%
-   */
-#if INT_MAX > COGL_SQRTI_ARG_10_PERCENT
-  if (t <= COGL_SQRTI_ARG_10_PERCENT)
-    return cogl_sqrti (t);
-  else
-    return COGL_FLOAT_TO_INT (sqrtf(t));
-#else
-  return cogl_sqrti (t);
-#endif
+  return sqrtf (t);
 }
 
 static void
