@@ -71,8 +71,9 @@
 #include "clutter-scriptable.h"
 #include "clutter-stage-private.h"
 
-#include "clutter-shader.h"
-#include "clutter-util.h"
+#include "deprecated/clutter-shader.h"
+#include "deprecated/clutter-texture.h"
+#include "deprecated/clutter-util.h"
 
 static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
@@ -1832,7 +1833,7 @@ texture_repaint_upload_func (gpointer user_data)
           clutter_texture_async_data_free (async_data);
         }
       while (upload_list != NULL &&
-             g_get_monotonic_time () < start_time + 5 * 1000000L);
+             g_get_monotonic_time () < start_time + 5 * 1000L);
     }
 
   if (upload_list != NULL)
