@@ -38,7 +38,7 @@
  * #ClutterShader is available since Clutter 0.6.
  *
  * #ClutterShader is deprecated since Clutter 1.8; use #ClutterShaderEffect
- * in newly written code, instead.
+ * in newly written code.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -71,6 +71,8 @@
 /* global list of shaders */
 static GList *clutter_shaders_list = NULL;
 
+#define CLUTTER_SHADER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_SHADER, ClutterShaderPrivate))
+
 struct _ClutterShaderPrivate
 {
   guint       compiled         : 1; /* Shader is bound to the GL context */
@@ -101,7 +103,7 @@ enum
 
 static GParamSpec *obj_props[PROP_LAST];
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterShader, clutter_shader, G_TYPE_OBJECT)
+G_DEFINE_TYPE (ClutterShader, clutter_shader, G_TYPE_OBJECT);
 
 static inline void
 clutter_shader_release_internal (ClutterShader *shader)
@@ -243,6 +245,8 @@ clutter_shader_class_init (ClutterShaderClass *klass)
   object_class->get_property  = clutter_shader_get_property;
   object_class->constructor   = clutter_shader_constructor;
 
+  g_type_class_add_private (klass, sizeof (ClutterShaderPrivate));
+
   /**
    * ClutterShader:vertex-source:
    *
@@ -319,7 +323,7 @@ clutter_shader_init (ClutterShader *self)
 {
   ClutterShaderPrivate *priv;
 
-  priv = self->priv = clutter_shader_get_instance_private (self);
+  priv = self->priv = CLUTTER_SHADER_GET_PRIVATE (self);
 
   priv->compiled = FALSE;
 
@@ -512,12 +516,12 @@ clutter_shader_glsl_bind (ClutterShader      *self,
 
       log_buf = cogl_shader_get_info_log (shader);
 
+      /* translators: the first %s is the type of the shader, either
+       * Vertex shader or Fragment shader; the second %s is the actual
+       * error as reported by COGL
+       */
       g_set_error (error, CLUTTER_SHADER_ERROR,
                    CLUTTER_SHADER_ERROR_COMPILE,
-                   /* translators: the first %s is the type of the shader, either
-                    * Vertex shader or Fragment shader; the second %s is the actual
-                    * error as reported by COGL
-                    */
                    _("%s compilation failed: %s"),
                    shader_type == CLUTTER_VERTEX_SHADER ? _("Vertex shader")
                                                         : _("Fragment shader"),
@@ -762,7 +766,7 @@ clutter_shader_set_uniform (ClutterShader *shader,
                             const GValue  *value)
 {
   ClutterShaderPrivate *priv;
-  int                   location = 0;
+  GLint                 location = 0;
   gsize                 size;
 
   g_return_if_fail (CLUTTER_IS_SHADER (shader));
@@ -781,7 +785,7 @@ clutter_shader_set_uniform (ClutterShader *shader,
 
   if (CLUTTER_VALUE_HOLDS_SHADER_FLOAT (value))
     {
-      const float *floats;
+      const GLfloat *floats;
 
       floats = clutter_value_get_shader_float (value, &size);
       cogl_program_set_uniform_float (priv->program,
@@ -797,7 +801,7 @@ clutter_shader_set_uniform (ClutterShader *shader,
     }
   else if (CLUTTER_VALUE_HOLDS_SHADER_MATRIX (value))
     {
-      const float *matrix;
+      const GLfloat *matrix;
 
       matrix = clutter_value_get_shader_matrix (value, &size);
       cogl_program_set_uniform_matrix (priv->program,
@@ -805,7 +809,7 @@ clutter_shader_set_uniform (ClutterShader *shader,
     }
   else if (G_VALUE_HOLDS_FLOAT (value))
     {
-      float float_val = g_value_get_float (value);
+      GLfloat float_val = g_value_get_float (value);
 
       cogl_program_set_uniform_float (priv->program,
                                       location, 1, 1, &float_val);
