@@ -26,9 +26,8 @@
 #include <glib/gi18n-lib.h>
 
 #include <string.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -37,13 +36,7 @@
 #include <gdk/gdk.h>
 #include <cogl/cogl.h>
 
-#ifndef GDK_WINDOWING_WIN32
-#include <sys/ioctl.h>
-#endif
-
-#if defined(GDK_WINDOWING_X11) && defined(COGL_HAS_XLIB_SUPPORT)
 #include <cogl/cogl-xlib.h>
-#endif
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -128,10 +121,9 @@ cogl_gdk_filter (GdkXEvent  *xevent,
 		 gpointer    data)
 {
 #ifdef GDK_WINDOWING_X11
-  ClutterBackend *backend = data;
   CoglFilterReturn ret;
 
-  ret = cogl_xlib_renderer_handle_event (backend->cogl_renderer, (XEvent *) xevent);
+  ret = cogl_xlib_handle_event ((XEvent*)xevent);
   switch (ret)
     {
     case COGL_FILTER_REMOVE:
@@ -142,8 +134,6 @@ cogl_gdk_filter (GdkXEvent  *xevent,
       return GDK_FILTER_CONTINUE;
     }
 #endif
-
-  return GDK_FILTER_CONTINUE;
 }
 
 static gboolean
@@ -168,10 +158,19 @@ _clutter_backend_gdk_post_parse (ClutterBackend  *backend,
 
   g_assert (backend_gdk->display != NULL);
 
+#ifdef GDK_WINDOWING_X11
+  if (GDK_IS_X11_DISPLAY (backend_gdk->display))
+    {
+      /* Cogl needs to know the Xlib display connection for
+	 CoglTexturePixmapX11 */
+      cogl_xlib_set_display (gdk_x11_display_get_xdisplay (backend_gdk->display));
+    }
+#endif
+
   backend_gdk->screen = gdk_display_get_default_screen (backend_gdk->display);
 
   /* add event filter for Cogl events */
-  gdk_window_add_filter (NULL, cogl_gdk_filter, backend_gdk);
+  gdk_window_add_filter (NULL, cogl_gdk_filter, NULL);
 
   clutter_backend_gdk_init_settings (backend_gdk);
 
@@ -211,7 +210,7 @@ clutter_backend_gdk_finalize (GObject *gobject)
 {
   ClutterBackendGdk *backend_gdk = CLUTTER_BACKEND_GDK (gobject);
 
-  gdk_window_remove_filter (NULL, cogl_gdk_filter, backend_gdk);
+  gdk_window_remove_filter (NULL, cogl_gdk_filter, NULL);
   g_object_unref (backend_gdk->display);
 
   G_OBJECT_CLASS (clutter_backend_gdk_parent_class)->finalize (gobject);
@@ -278,8 +277,7 @@ clutter_backend_gdk_get_renderer (ClutterBackend  *backend,
   if (GDK_IS_WIN32_DISPLAY (backend_gdk->display))
     {
       /* Force a WGL winsys on windows */
-      cogl_renderer_set_winsys_id (renderer, COGL_WINSYS_ID_WGL);
-      cogl_win32_renderer_set_event_retrieval_enabled (renderer, FALSE);
+      cogl_renderer_set_winsys_id (backend_cogl->cogl_renderer, COGL_WINSYS_ID_WGL);
     }
   else
 #endif
