@@ -154,24 +154,93 @@ translate_device_classes (Display             *xdisplay,
                                     (XIValuatorClassInfo *) class_info);
           break;
 
+#ifdef HAVE_XINPUT_2_2
+        case XIScrollClass:
+          {
+            XIScrollClassInfo *scroll_info = (XIScrollClassInfo *) class_info;
+            ClutterScrollDirection direction;
+
+            if (scroll_info->scroll_type == XIScrollTypeVertical)
+              direction = CLUTTER_SCROLL_DOWN;
+            else
+              direction = CLUTTER_SCROLL_RIGHT;
+
+            CLUTTER_NOTE (BACKEND, "Scroll valuator %d: %s, increment: %f",
+                          scroll_info->number,
+                          scroll_info->scroll_type == XIScrollTypeVertical
+                            ? "vertical"
+                            : "horizontal",
+                          scroll_info->increment);
+
+            _clutter_input_device_add_scroll_info (device,
+                                                   scroll_info->number,
+                                                   direction,
+                                                   scroll_info->increment);
+          }
+          break;
+#endif /* HAVE_XINPUT_2_2 */
+
         default:
           break;
         }
     }
 }
 
+static gboolean
+is_touch_device (XIAnyClassInfo         **classes,
+                 guint                    n_classes,
+                 ClutterInputDeviceType  *device_type,
+                 guint                   *n_touch_points)
+{
+#ifdef HAVE_XINPUT_2_2
+  guint i;
+
+  for (i = 0; i < n_classes; i++)
+    {
+      XITouchClassInfo *class = (XITouchClassInfo *) classes[i];
+
+      if (class->type != XITouchClass)
+        continue;
+
+      if (class->num_touches > 0)
+        {
+          if (class->mode == XIDirectTouch)
+            *device_type = CLUTTER_TOUCHSCREEN_DEVICE;
+          else if (class->mode == XIDependentTouch)
+            *device_type = CLUTTER_TOUCHPAD_DEVICE;
+          else
+            continue;
+
+          *n_touch_points = class->num_touches;
+
+          return TRUE;
+        }
+    }
+#endif
+
+  return FALSE;
+}
+
 static ClutterInputDevice *
 create_device (ClutterDeviceManagerXI2 *manager_xi2,
                ClutterBackendX11       *backend_x11,
                XIDeviceInfo            *info)
 {
-  ClutterInputDeviceType source;
+  ClutterInputDeviceType source, touch_source;
   ClutterInputDevice *retval;
   ClutterInputMode mode;
   gboolean is_enabled;
+  guint num_touches = 0;
 
   if (info->use == XIMasterKeyboard || info->use == XISlaveKeyboard)
     source = CLUTTER_KEYBOARD_DEVICE;
+  else if (info->use == XISlavePointer &&
+           is_touch_device (info->classes, info->num_classes,
+                            &touch_source,
+                            &num_touches))
+    {
+      source = touch_source;
+    }
   else
     {
       gchar *name;
@@ -414,6 +483,11 @@ get_event_stage (ClutterEventTranslator *translator,
     case XI_ButtonPress:
     case XI_ButtonRelease:
     case XI_Motion:
+#ifdef HAVE_XINPUT_2_2
+    case XI_TouchBegin:
+    case XI_TouchUpdate:
+    case XI_TouchEnd:
+#endif /* HAVE_XINPUT_2_2 */
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
 
@@ -541,6 +615,51 @@ translate_axes (ClutterInputDevice *device,
   return retval;
 }
 
+static gdouble
+scroll_valuators_changed (ClutterInputDevice *device,
+                          XIValuatorState    *valuators,
+                          gdouble            *dx_p,
+                          gdouble            *dy_p)
+{
+  gboolean retval = FALSE;
+  guint n_axes, n_val, i;
+  double *values;
+
+  n_axes = clutter_input_device_get_n_axes (device);
+  values = valuators->values;
+
+  *dx_p = *dy_p = 0.0;
+
+  n_val = 0;
+
+  for (i = 0; i < MIN (valuators->mask_len * 8, n_axes); i++)
+    {
+      ClutterScrollDirection direction;
+      gdouble delta;
+
+      if (!XIMaskIsSet (valuators->mask, i))
+        continue;
+
+      if (_clutter_input_device_get_scroll_delta (device, i,
+                                                  values[n_val],
+                                                  &direction,
+                                                  &delta))
+        {
+          retval = TRUE;
+
+          if (direction == CLUTTER_SCROLL_UP ||
+              direction == CLUTTER_SCROLL_DOWN)
+            *dx_p = delta;
+          else
+            *dy_p = delta;
+        }
+
+      n_val += 1;
+    }
+
+  return retval;
+}
+
 static ClutterTranslateReturn
 clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             gpointer                native,
@@ -601,6 +720,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         device = g_hash_table_lookup (manager_xi2->devices_by_id,
                                       GINT_TO_POINTER (xev->deviceid));
         _clutter_input_device_reset_axes (device);
+        _clutter_input_device_reset_scroll_info (device);
         translate_device_classes (backend_x11->xdpy,
                                   device,
                                   xev->classes,
@@ -735,6 +855,10 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                                  stage_x11,
                                                  &xev->valuators);
 
+#ifdef HAVE_XINPUT_2_2
+            if (xev->flags & XIPointerEmulated)
+              _clutter_event_set_pointer_emulated (event, TRUE);
+#endif /* HAVE_XINPUT_2_2 */
             break;
 
           default:
@@ -783,6 +907,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                       event->button.y,
                       event->button.axes != NULL ? "yes" : "no");
 
+#ifdef HAVE_XINPUT_2_2
+        if (xev->flags & XIPointerEmulated)
+          _clutter_event_set_pointer_emulated (event, TRUE);
+#endif /* HAVE_XINPUT_2_2 */
+
         if (xi_event->evtype == XI_ButtonPress)
           _clutter_stage_x11_set_user_time (stage_x11, event->button.time);
 
@@ -793,6 +922,44 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
     case XI_Motion:
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
+        gdouble delta_x, delta_y;
+
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
+
+        if (scroll_valuators_changed (source_device,
+                                      &xev->valuators,
+                                      &delta_x, &delta_y))
+          {
+            event->scroll.type = event->type = CLUTTER_SCROLL;
+            event->scroll.direction = CLUTTER_SCROLL_SMOOTH;
+
+            event->scroll.stage = stage;
+            event->scroll.time = xev->time;
+            event->scroll.x = xev->event_x;
+            event->scroll.y = xev->event_y;
+            event->scroll.modifier_state =
+              _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                         &xev->buttons);
+
+            clutter_event_set_scroll_delta (event, delta_x, delta_y);
+            clutter_event_set_source_device (event, source_device);
+
+            device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                          GINT_TO_POINTER (xev->deviceid));
+            clutter_event_set_device (event, device);
+
+            CLUTTER_NOTE (EVENT,
+                          "smooth scroll: win:0x%x device:%s (x:%.2f, y:%.2f, delta:%f, %f)",
+                          (unsigned int) stage_x11->xwin,
+                          event->scroll.device->device_name,
+                          event->scroll.x,
+                          event->scroll.y,
+                          delta_x, delta_y);
+
+            retval = CLUTTER_TRANSLATE_QUEUE;
+            break;
+          }
 
         event->motion.type = event->type = CLUTTER_MOTION;
 
@@ -805,8 +972,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
           _clutter_input_device_xi2_translate_state (&xev->mods,
                                                      &xev->buttons);
 
-        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                             GINT_TO_POINTER (xev->sourceid));
         clutter_event_set_source_device (event, source_device);
 
         device = g_hash_table_lookup (manager_xi2->devices_by_id,
@@ -822,6 +987,11 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         if (source_device != NULL && device->stage != NULL)
           _clutter_input_device_set_stage (source_device, device->stage);
 
+#ifdef HAVE_XINPUT_2_2
+        if (xev->flags & XIPointerEmulated)
+          _clutter_event_set_pointer_emulated (event, TRUE);
+#endif /* HAVE_XINPUT_2_2 */
+
         CLUTTER_NOTE (EVENT, "motion: win:0x%x device:%s (x:%.2f, y:%.2f, axes:%s)",
                       (unsigned int) stage_x11->xwin,
                       event->motion.device->device_name,
@@ -833,6 +1003,116 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
       }
       break;
 
+#ifdef HAVE_XINPUT_2_2
+    case XI_TouchBegin:
+    case XI_TouchEnd:
+      {
+        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
+
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
+
+        if (xi_event->evtype == XI_TouchBegin)
+          event->touch.type = event->type = CLUTTER_TOUCH_BEGIN;
+        else
+          event->touch.type = event->type = CLUTTER_TOUCH_END;
+
+        event->touch.stage = stage;
+        event->touch.time = xev->time;
+        event->touch.x = xev->event_x;
+        event->touch.y = xev->event_y;
+        event->touch.modifier_state =
+          _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                     &xev->buttons);
+
+        clutter_event_set_source_device (event, source_device);
+
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
+        clutter_event_set_device (event, device);
+
+        event->touch.axes = translate_axes (event->motion.device,
+                                            event->motion.x,
+                                            event->motion.y,
+                                            stage_x11,
+                                            &xev->valuators);
+
+        if (source_device != NULL && device->stage != NULL)
+          _clutter_input_device_set_stage (source_device, device->stage);
+
+        if (xi_event->evtype == XI_TouchBegin)
+          {
+            event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
+
+            _clutter_stage_x11_set_user_time (stage_x11, event->touch.time);
+          }
+
+        event->touch.sequence = GUINT_TO_POINTER (xev->detail);
+
+        if (xev->flags & XITouchEmulatingPointer)
+          _clutter_event_set_pointer_emulated (event, TRUE);
+
+        CLUTTER_NOTE (EVENT, "touch %s: win:0x%x device:%s (x:%.2f, y:%.2f, axes:%s)",
+                      event->type == CLUTTER_TOUCH_BEGIN ? "begin" : "end",
+                      (unsigned int) stage_x11->xwin,
+                      event->touch.device->device_name,
+                      event->touch.x,
+                      event->touch.y,
+                      event->touch.axes != NULL ? "yes" : "no");
+
+        retval = CLUTTER_TRANSLATE_QUEUE;
+      }
+      break;
+
+    case XI_TouchUpdate:
+      {
+        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
+
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
+
+        event->touch.type = event->type = CLUTTER_TOUCH_UPDATE;
+        event->touch.stage = stage;
+        event->touch.time = xev->time;
+        event->touch.sequence = GUINT_TO_POINTER (xev->detail);
+        event->touch.x = xev->event_x;
+        event->touch.y = xev->event_y;
+
+        clutter_event_set_source_device (event, source_device);
+
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
+        clutter_event_set_device (event, device);
+
+        event->touch.axes = translate_axes (event->motion.device,
+                                            event->motion.x,
+                                            event->motion.y,
+                                            stage_x11,
+                                            &xev->valuators);
+
+        if (source_device != NULL && device->stage != NULL)
+          _clutter_input_device_set_stage (source_device, device->stage);
+
+        event->touch.modifier_state =
+          _clutter_input_device_xi2_translate_state (&xev->mods,
+                                                     &xev->buttons);
+        event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
+
+        if (xev->flags & XITouchEmulatingPointer)
+          _clutter_event_set_pointer_emulated (event, TRUE);
+
+        CLUTTER_NOTE (EVENT, "touch update: win:0x%x device:%s (x:%.2f, y:%.2f, axes:%s)",
+                      (unsigned int) stage_x11->xwin,
+                      event->touch.device->device_name,
+                      event->touch.x,
+                      event->touch.y,
+                      event->touch.axes != NULL ? "yes" : "no");
+
+        retval = CLUTTER_TRANSLATE_QUEUE;
+      }
+      break;
+#endif /* HAVE_XINPUT_2_2 */
+
     case XI_Enter:
     case XI_Leave:
       {
@@ -883,6 +1163,8 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             _clutter_stage_remove_device (stage, device);
           }
 
+        _clutter_input_device_reset_scroll_info (source_device);
+
         clutter_event_set_device (event, device);
         clutter_event_set_source_device (event, source_device);
 
